/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//// This example is from the ACE Programmers Guide.
////  Chapter:  "Thread Management"
//// For details please see the guide at
//// http://www.cs.wustl.edu/~schmidt/ACE.html
////  AUTHOR: Umar Syyid (usyyid@hns.com)
//// and Ambreen Ilyas (ambreen@bitsmart.com)
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

//Example 4
#include "ace/OS.h"
#include "ace/Token.h"

//Arguments that are to be passed to the worker thread are passed
//through this class.
class Args{
public:
Args(int iterations):
 mutex_(),iterations_(iterations){}
ACE_Token mutex_;
int iterations_;
};

//The starting point for the worker threads
static void*
worker(void*arguments){
Args *arg= (Args*) arguments;
for(int i=0;i<arg->iterations_;i++){
 ACE_DEBUG((LM_DEBUG,"(%t) Trying to get a hold of this iteration\n"));
 //This is our critical section
 arg->mutex_.acquire();
 ACE_DEBUG((LM_DEBUG,"(%t) This is iteration number %d\n",i));
 //work
 ACE_OS::sleep(2);
 arg->mutex_.release();
 }
return 0;
}
 

int main(int argc, char*argv[]){
  if(argc<3){
    ACE_OS::printf("Usage: %s <number_of_threads> <number_of_iterations>\n", argv[0]);
    ACE_OS::exit(1);
 }

Args arg(ACE_OS::atoi(argv[2]));
//Setup the arguments

int n_threads = ACE_OS::atoi(argv[1]);
//determine the number of threads to be spawned

ACE_thread_t *threadID= new ACE_thread_t[n_threads+1];
ACE_hthread_t *threadHandles = new ACE_hthread_t[n_threads+1];

if (ACE_Thread::spawn_n(threadID, n_threads, (ACE_THR_FUNC)worker, &arg, THR_JOINABLE | THR_NEW_LWP,
			ACE_DEFAULT_THREAD_PRIORITY, 0, 0, threadHandles) == -1)
  ACE_DEBUG ((LM_DEBUG,"Error in spawning thread\n"));
// spawn n_threads
  

for (int i=0; i<n_threads; i++)
  ACE_Thread::join(threadHandles[i]);

//Wait for all threads to exit before you let the main fall through
//and have the process exit.

return 0;	
} 
